from ftw.events.behaviors.location import ILocationFields
from ftw.events.interfaces import IEventPage
from plone.app.event.dx.behaviors import IEventBasic
from plone.app.event.dx.behaviors import IEventLocation
from plone.autoform.interfaces import IFormFieldProvider
from plone.dexterity.content import Container
from plone.directives import form
from plone.supermodel.interfaces import FIELDSETS_KEY
from zope.interface import alsoProvides
from zope.interface import implements


# Move plone.app.event's default field "timezone" to the default schema,
# since all other fields are in the default schema and we do not want
# an extra schema only for the "timezone" field.
for fieldset in IEventBasic.queryTaggedValue(FIELDSETS_KEY, []):
    if 'timezone' in fieldset.fields:
        fieldset.fields.remove('timezone')


class IEventPageSchema(form.Schema):
    pass

alsoProvides(IEventPageSchema, IFormFieldProvider)


class EventPage(Container):
    implements(IEventPage)

    @property
    def location(self):
        if ILocationFields.providedBy(self):
            storage = ILocationFields(self)
            return ', '.join(filter(None, (
                storage.location_title,
                storage.location_street,
                ' '.join((storage.location_zip or '',
                          storage.location_city or '')).strip(),
            ))) or ''

        elif IEventLocation.providedBy(self):
            return vars(IEventLocation(self)).get('location', None)

        else:
            return None



from datetime import datetime
from ftw.builder import Builder
from ftw.builder import create
from ftw.events.tests import FunctionalTestCase
from ftw.testbrowser import browsing
from ftw.testbrowser.pages import factoriesmenu
from ftw.testbrowser.pages import statusmessages


class TestEventPage(FunctionalTestCase):

    @browsing
    def test_create_event_page(self, browser):
        self.grant('Manager')
        folder = create(Builder('event folder').titled(u'Activities'))
        browser.login().open(folder)
        factoriesmenu.add('Event Page')

        browser.fill({'Title': 'Jogging'}).save()
        statusmessages.assert_no_error_messages()
        self.assertEquals('http://nohost/plone/activities/jogging/view',
                          browser.url)

    @browsing
    def test_event_details_WHEN_shows_dates(self, browser):
        self.grant('Manager')
        folder = create(Builder('event folder'))
        event = create(Builder('event page')
                       .starting(datetime(2013, 10, 7, 9, 00))
                       .ending(datetime(2013, 10, 7, 16, 00))
                       .having(recurrence='RRULE:FREQ=DAILY;COUNT=4')
                       .within(folder))

        browser.login().open(event)
        self.assertEquals(['Oct 07, 2013 from 09:00 AM to 04:00 PM',
                           'Oct 08, 2013 from 09:00 AM to 04:00 PM',
                           'Oct 09, 2013 from 09:00 AM to 04:00 PM',
                           'Oct 10, 2013 from 09:00 AM to 04:00 PM'],
                          browser.css('.event-details .when li').text)

    @browsing
    def test_event_details_shows_location(self, browser):
        self.grant('Manager')
        folder = create(Builder('event folder'))
        event = create(Builder('event page')
                       .having(location_title='Infinite Loop 1')
                       .within(folder))

        browser.login().open(event)
        self.assertEqual(
            ['Infinite Loop 1'],
            browser.css('.event-details .location p').text
        )



from Acquisition import aq_chain
from DateTime import DateTime
from ftw.events import _
from ftw.events.interfaces import IEventPage
from plone.app.dexterity.behaviors.metadata import DCFieldProperty
from plone.app.dexterity.behaviors.metadata import MetadataBase
from plone.autoform.directives import read_permission
from plone.autoform.directives import write_permission
from plone.directives.form import fieldset
from plone.directives.form import IFormFieldProvider
from plone.directives.form import Schema
from Products.Five.browser import BrowserView
from zope import schema
from zope.annotation import IAnnotations
from zope.component.hooks import getSite
from zope.interface import alsoProvides
from zope.interface import implements
from zope.interface import Interface
import requests
import urllib
import urlparse


class IPublisherMopageTrigger(Schema):

    fieldset('mopage',
             label=_(u'Mopage'),
             fields=['mopage_enabled',
                     'mopage_trigger_url',
                     'mopage_data_endpoint_url'])

    read_permission(mopage_enabled='ftw.events.ConfigureMopageTrigger')
    write_permission(mopage_enabled='ftw.events.ConfigureMopageTrigger')
    mopage_enabled = schema.Bool(
        title=_(u'label_trigger_enabled',
                default=u'Mopage trigger enabled'),
        default=False,
        required=False,
    )

    read_permission(mopage_trigger_url='ftw.events.ConfigureMopageTrigger')
    write_permission(mopage_trigger_url='ftw.events.ConfigureMopageTrigger')
    mopage_trigger_url = schema.URI(
        title=_(u'label_mopage_trigger_url',
                default=u'Mopage trigger URL'),
        description=_(
            u'description_mopage_trigger_url',
            default=u'Contains the mopage URL to the trigger endpoint.'
            u' This is only the base URL, it does not contain the endpoint URL'
            u' from which the mopage server retrieves the events.'
            u' Example: https://un:pw@xml.mopage.ch/infoservice/xml.php'
        ),
        default=None,
        required=False,
    )

    read_permission(
        mopage_data_endpoint_url='ftw.events.ConfigureMopageTrigger')
    write_permission(
        mopage_data_endpoint_url='ftw.events.ConfigureMopageTrigger')
    mopage_data_endpoint_url = schema.URI(
        title=_(u'label_mopage_data_endpoint_url',
                default=u'Mopage data endpoint URL (Plone)'),
        description=_(
            u'description_mopage_data_endpoint_url',
            default=u'The mopage data endpoint URL points to the'
            u' "mopage.events.xml" view somewhere on the public visible '
            u' Plone page. It must also contain the params "partnerid"'
            u' and "importid".'
            u' Example: https://mypage.ch/events/'
            u'mopage.events.xml?partnerid=3&importid=6'
        ),
        default=None,
        required=False,
    )


alsoProvides(IPublisherMopageTrigger, IFormFieldProvider)


class IPublisherMopageTriggerSupport(Interface):
    """Marker interface for event folders which support IPublisherMopageTrigger.
    """


class PublisherMopageTrigger(MetadataBase):
    """Behavior adapter for IPublisherMopageTrigger.
    The storage is the adapted context.
    """

    mopage_enabled = DCFieldProperty(
        IPublisherMopageTrigger['mopage_enabled'])
    mopage_trigger_url = DCFieldProperty(
        IPublisherMopageTrigger['mopage_trigger_url'])
    mopage_data_endpoint_url = DCFieldProperty(
        IPublisherMopageTrigger['mopage_data_endpoint_url'])

    def is_enabled(self):
        return self.mopage_enabled \
            and self.mopage_trigger_url \
            and self.mopage_data_endpoint_url

    def build_trigger_url(self):
        if not self.is_enabled():
            return None

        parts = list(urlparse.urlparse(self.mopage_trigger_url))
        parts[5] = ''  # drop anchor
        parts[4] = urllib.urlencode({'url': self.mopage_data_endpoint_url})
        return urlparse.urlunparse(parts)


class IMopageModificationDate(Interface):
    """The modification date adapter stores and provides a mopage
    specific modification date which is used for tracking content changes
    of an event page.
    This is important in order to tell the Mopage system that something has
    changed.
    """

    def get_date():
        """Return the current modification date.
        """

    def set_date(date):
        """Set modification date to a speicifc date.
        """

    def touch():
        """Set modification date to now.
        """


class MopageModificationDate(object):
    implements(IMopageModificationDate)

    annotations_key = 'ftw.events:mopage:modification_date'

    def __init__(self, context):
        self.context = context

    def get_date(self):
        date = IAnnotations(self.context).get(self.annotations_key, None)
        return date or self.context.modified()

    def set_date(self, date):
        IAnnotations(self.context)[self.annotations_key] = DateTime(date)

    def touch(self):
        self.set_date(DateTime())


def trigger_mopage_refresh(obj, event):
    event_pages = filter(None,
                          map(lambda parent: IEventPage(parent, None),
                              aq_chain(obj)))
    if not event_pages:
        # We are not within an event page.
        # We only trigger when publishing an event page
        # or a child of an event page.
        return

    triggers = filter(None,
                      map(lambda parent: IPublisherMopageTrigger(parent, None),
                          aq_chain(obj)))
    if not triggers or not triggers[0].is_enabled():
        return

    for events in event_pages:
        IMopageModificationDate(events).touch()

    from collective.taskqueue import taskqueue

    trigger_url = triggers[0].build_trigger_url()
    callback_path = '/'.join(getSite().getPhysicalPath()
                             + ('taskqueue_events_trigger_mopage_refresh',))
    taskqueue.add(callback_path, params={'target': trigger_url})


class TriggerMopageRefreshTaskQueueWorker(BrowserView):

    def __call__(self):
        requests.get(self.request.form['target']).raise_for_status()



from ftw.events.tests import FunctionalTestCase
from Products.CMFCore.utils import getToolByName


class TestDefaultProfile(FunctionalTestCase):

    def test_installed(self):
        portal_setup = getToolByName(self.portal, 'portal_setup')
        version = portal_setup.getLastVersionForProfile('ftw.events:default')
        self.assertNotEqual(version, None)
        self.assertNotEqual(version, 'unknown')



from ftw.events.interfaces import IEventListingBlock
from plone import api
from plone.app.layout.links.viewlets import RSSViewlet
from Products.CMFPlone.interfaces.syndication import IFeedSettings
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile


class EventFolderRSSViewlet(RSSViewlet):
    """
    This custom RSS viewlet renders the RSS links in the head tag
    of an event folder for all its event listing blocks having
    syndication enabled.
    """
    index = ViewPageTemplateFile('templates/rsslink.pt')

    def update(self):
        super(EventFolderRSSViewlet, self).update()
        self.rsslinks = []
        for block in self.get_blocks_with_syndication_enabled():
            self.rsslinks.extend(self.getRssLinks(block))

    def get_blocks_with_syndication_enabled(self):
        blocks = api.content.find(
            context=self.context,
            object_provides=IEventListingBlock.__identifier__,
        )
        blocks = [brain.getObject() for brain in blocks]
        blocks = filter(
            lambda block: IFeedSettings(block).enabled,
            blocks
        )
        return blocks



from collective.taskqueue.testing import runAsyncTest
from DateTime import DateTime
from datetime import datetime
from ftw.builder import Builder
from ftw.builder import create
from ftw.events.behaviors.mopage import IMopageModificationDate
from ftw.events.behaviors.mopage import IPublisherMopageTrigger
from ftw.events.testing import MOPAGE_TRIGGER_FUNCTIONAL
from ftw.events.tests import FunctionalTestCase
from ftw.publisher.receiver.events import AfterCreatedEvent
from ftw.publisher.receiver.events import AfterUpdatedEvent
from ftw.testbrowser import browsing
from ftw.testbrowser.pages import factoriesmenu
from ftw.testbrowser.pages import statusmessages
from ftw.testing import freeze
from persistent.list import PersistentList
from Products.CMFCore.utils import getToolByName
from Products.Five.browser import BrowserView
from pytz import timezone
from zope.annotation.interfaces import IAnnotations
from zope.event import notify
import transaction
import urlparse


def get_stub_log(portal):
    annotations = IAnnotations(portal)
    key = 'ftw.events-mopage-stub-log'
    if key not in annotations:
        annotations[key] = PersistentList()
    return annotations[key]


class MopageAPIStub(BrowserView):

    def __call__(self):
        log = get_stub_log(self.context)
        log.append(self.request.form)
        return 'OK'


class TestMopageTrigger(FunctionalTestCase):
    layer = MOPAGE_TRIGGER_FUNCTIONAL

    def setUp(self):
        super(TestMopageTrigger, self).setUp()
        portal_types = getToolByName(self.portal, 'portal_types')
        portal_types['ftw.events.EventFolder'].behaviors += (
            'ftw.events.behaviors.mopage.IPublisherMopageTrigger',
        )
        transaction.commit()

    @browsing
    def test_configure_trigger_on_event_folder(self, browser):
        self.grant('Manager')
        browser.login().open()
        factoriesmenu.add('Event Folder')
        browser.fill(
            {'Title': 'Event Folder',
             'Mopage trigger enabled': True,
             'Mopage trigger URL': (
                 'https://un:pw@xml.mopage.ch/infoservice/xml.php'),
             'Mopage data endpoint URL (Plone)': (
                 'http://nohost/plone/event-folder/mopage.events.xml'
                 '?partnerid=123&imported=456')}).save()

        statusmessages.assert_no_error_messages()

        mopage_trigger = IPublisherMopageTrigger(browser.context)
        self.assertTrue(mopage_trigger.is_enabled())

        trigger_url = mopage_trigger.build_trigger_url()
        self.assertEquals('https://un:pw@xml.mopage.ch/infoservice/xml.php'
                          '?url=http%3A%2F%2Fnohost%2Fplone%2Fevent-folder'
                          '%2Fmopage.events.xml%3Fpartnerid%3D123%26imported%3D456',
                          trigger_url)

        params =  urlparse.parse_qs(urlparse.urlparse(trigger_url)[4])['url']
        self.assertEquals(['http://nohost/plone/event-folder/'
                           'mopage.events.xml?partnerid=123&imported=456'],
                          params)

    def test_trigger_notified_when_events_created(self):
        self.grant('Manager')
        trigger_url = self.portal.portal_url() + '/mopage-stub'
        endpoint_url = (self.portal.portal_url() + '/event-folder/mopage.events.xml' +
                        '?partnerid=213&importid=456')

        folder = create(Builder('event folder')
                        .titled(u'Event Folder')
                        .having(mopage_enabled=True,
                                mopage_trigger_url=trigger_url,
                                mopage_data_endpoint_url=endpoint_url))

        events = create(Builder('event page').titled(u'The Events').within(folder))
        self.assertEquals([], get_stub_log(self.portal))

        notify(AfterCreatedEvent(events))
        transaction.commit()
        runAsyncTest(lambda: None)
        transaction.begin()
        self.assertEquals(
            [{'url': endpoint_url}],
            get_stub_log(self.portal))

    def test_trigger_notified_when_events_updated(self):
        self.grant('Manager')
        trigger_url = self.portal.portal_url() + '/mopage-stub'
        endpoint_url = (self.portal.portal_url() + '/event-folder/mopage.events.xml' +
                        '?partnerid=999&importid=888')

        folder = create(Builder('event folder')
                        .titled(u'Event Folder')
                        .having(mopage_enabled=True,
                                mopage_trigger_url=trigger_url,
                                mopage_data_endpoint_url=endpoint_url))

        events = create(Builder('event page').titled(u'The Events').within(folder))
        self.assertEquals([], get_stub_log(self.portal))

        notify(AfterUpdatedEvent(events))
        transaction.commit()
        runAsyncTest(lambda: None)
        transaction.begin()
        self.assertEquals(
            [{'url': endpoint_url}],
            get_stub_log(self.portal))

    def test_trigger_notified_when_sl_block_is_created_in_a_events(self):
        """When a events is published, the first page is the events, then
        the content of the events follows in separate jobs (sl blocks,
        recursive structures).
        When we only trigger when the events itself is published, the content
        will be missing.
        Therefore we must trigger an update on any change within the events.
        """
        self.grant('Manager')
        trigger_url = self.portal.portal_url() + '/mopage-stub'
        endpoint_url = (self.portal.portal_url() + '/event-folder/mopage.events.xml' +
                        '?partnerid=213&importid=456')

        folder = create(Builder('event folder')
                        .titled(u'Event Folder')
                        .having(mopage_enabled=True,
                                mopage_trigger_url=trigger_url,
                                mopage_data_endpoint_url=endpoint_url))

        events = create(Builder('event page').titled(u'The Events').within(folder))
        block = create(Builder('sl textblock').within(events))

        self.assertEquals([], get_stub_log(self.portal))

        notify(AfterCreatedEvent(block))
        transaction.commit()
        runAsyncTest(lambda: None)
        transaction.begin()
        self.assertEquals(
            [{'url': endpoint_url}],
            get_stub_log(self.portal))

    def test_publisher_event_updates_mopage_modified_date(self):
        """The idea of the mopage modified date is that it tracks modifications
        on the content, more specific blocks added by publisher.
        This is important in order to tell the mopage system that things have
        changed.
        However, it is not the Plone modification date and it should be updated
        by the publisher subscriber.
        """
        self.grant('Manager')
        trigger_url = self.portal.portal_url() + '/mopage-stub'
        endpoint_url = (self.portal.portal_url() + '/event-folder/mopage.events.xml' +
                        '?partnerid=213&importid=456')
        folder = create(Builder('event folder')
                        .having(mopage_enabled=True,
                                mopage_trigger_url=trigger_url,
                                mopage_data_endpoint_url=endpoint_url))

        zurich = timezone('Europe/Zurich')
        with freeze(datetime(2016, 1, 1, tzinfo=zurich)) as clock:
            events = create(Builder('event page').within(folder))
            block = create(Builder('sl textblock').within(events))
            self.assertEquals(DateTime('2016/01/01 01:00:00 GMT+1'),
                              IMopageModificationDate(events).get_date())

            clock.forward(days=1)
            notify(AfterCreatedEvent(block))
            self.assertEquals(DateTime('2016/01/02 00:00:00 GMT+1'),
                              IMopageModificationDate(events).get_date())

            clock.forward(days=1)
            events.setModificationDate(DateTime())
            notify(AfterCreatedEvent(block))
            self.assertEquals(DateTime('2016/01/03 00:00:00 GMT+1'),
                              IMopageModificationDate(events).get_date())



from ftw.events import _
from plone.directives.form import IFormFieldProvider
from plone.directives.form import Schema
from zope.interface import alsoProvides
from zope.schema import TextLine


class ILocationFields(Schema):

    location_title = TextLine(
        title=_(
            u'label_location_title',
            default=u'Location: title'
        ),
        required=False,
        default=None,
    )

    location_street = TextLine(
        title=_(
            u'label_location_street',
            default=u'Location: street and number'
        ),
        required=False,
        default=None,
    )

    location_zip = TextLine(
        title=_(
            u'label_location_zip',
            default=u'Location: ZIP code'
        ),
        required=False,
        default=None,
    )

    location_city = TextLine(
        title=_(
            u'label_location_city',
            default=u'Location: city'
        ),
        required=False,
        default=None,
    )

alsoProvides(ILocationFields, IFormFieldProvider)



import datetime
from ftw.builder import Builder
from ftw.builder import create
from ftw.events.tests import FunctionalTestCase
from ftw.testbrowser import browsing
from ftw.testbrowser.pages import factoriesmenu
from ftw.testbrowser.pages import statusmessages
from ftw.testing import freeze
from Products.CMFPlone.interfaces.syndication import IFeedSettings


class TestEventListingBlock(FunctionalTestCase):

    def setUp(self):
        super(TestEventListingBlock, self).setUp()
        self.grant('Manager')

    def _get_event_titles_from_block(self, browser):
        """
        This helper method returns the titles of the events rendered
        in the event listing block. Use this method in the test case to
        make sure the block lists only the events we want it to.
        """
        return browser.css('.ftw-events-eventlistingblock .event-row h3.title').text

    @browsing
    def test_event_listing_block_builder(self, browser):
        page = create(Builder('sl content page').titled(u'A page'))

        create(Builder('event listing block')
               .within(page)
               .titled(u'This block renders event pages'))

        browser.login().visit(page)
        self.assertEqual(
            ['This block renders event pages'],
            browser.css('.sl-block.ftw-events-eventlistingblock h2').text
        )

    @browsing
    def test_event_listing_block_can_be_added_on_contentpage(self, browser):
        page = create(Builder('sl content page').titled(u'A page'))
        browser.login().visit(page)

        factoriesmenu.add('EventListingBlock')
        browser.fill({
            'Title': u'This block renders event pages',
        }).save()

        browser.open(page)
        self.assertEqual(
            ['This block renders event pages'],
            browser.css('.sl-block.ftw-events-eventlistingblock h2').text
        )

    @browsing
    def test_event_listing_block_does_not_render_title(self, browser):
        page = create(Builder('sl content page').titled(u'A page'))

        block = create(Builder('event listing block')
                       .within(page)
                       .titled(u'This block renders event pages'))
        browser.login()

        # Make sure the title is rendered by default.
        browser.visit(page)
        self.assertEqual(
            ['This block renders event pages'],
            browser.css('.sl-block.ftw-events-eventlistingblock h2').text
        )

        # Tell the block to no longer display the title.
        browser.login().visit(block, view='edit.json')
        response = browser.json
        browser.open_html(response['content'])
        browser.fill({
            'Show title': False,
        })
        browser.find_button_by_label('Save').click()

        # Make sure the title is no longer rendered.
        browser.visit(page)
        self.assertEqual(
            [],
            browser.css('.sl-block.ftw-events-eventlistingblock h2')
        )

    @browsing
    def test_sort_order(self, browser):
        """
        This test makes sure that the events are sorted in ascending order by
        their start date.
        """
        event_folder = create(Builder('event folder').titled(u'Event Folder'))
        with freeze(datetime.datetime(2010, 7, 1)):
            create(Builder('event page')
                   .titled(u'Event of Hans')
                   .starting(datetime.datetime(2010, 7, 2))
                   .ending(datetime.datetime(2010, 7, 3))
                   .within(event_folder))
            create(Builder('event page')
                   .titled(u'Event of Peter')
                   .starting(datetime.datetime(2010, 7, 1))
                   .ending(datetime.datetime(2010, 7, 5))
                   .within(event_folder))

            browser.login()

            # Make sure that the events are sorted in ascending order.
            browser.visit(event_folder)
            self.assertEqual(
                ['Event of Peter', 'Event of Hans'],
                self._get_event_titles_from_block(browser),
                "Event of Peter must be listed before the event of Hans"
            )

            # Folder listing still renders the items in the order they were added by.
            browser.visit(event_folder, view='folder_contents')
            self.assertEqual(
                ['Event of Hans', 'Event of Peter'],
                browser.css('table#listing-table tr td a.contenttype-ftw-events-eventpage').text
            )

    @browsing
    def test_block_filters_by_path(self, browser):
        """
        This test makes sure that the block only renders items
        from the given path.
        """
        event_folder1 = create(Builder('event folder')
                               .titled(u'Event Folder 1')
                               .within(self.portal))
        create(Builder('event page').titled(u'Hello World 1').within(event_folder1))

        event_folder2 = create(Builder('event folder')
                               .titled(u'Event Folder 2')
                               .within(self.portal))
        create(Builder('event page').titled(u'Hello World 2').within(event_folder2))

        # Prepare a page containing an event listing block.
        page = create(Builder('sl content page').titled(u'Content Page with Eventlistingblock'))
        block = create(Builder('event listing block')
                       .within(page)
                       .having(current_context=False)
                       .titled(u'Not relevant in this test'))

        browser.login()

        # Edit the block and set a path (not possible with builder).
        browser.visit(block, view='edit.json')
        response = browser.json
        browser.open_html(response['content'])
        browser.fill({
            'Limit to path': event_folder2,
        })
        browser.find_button_by_label('Save').click()
        browser.visit(page)

        # Make sure the block only lists events from the path
        # we have configured on the block.
        self.assertEqual(
            ['Hello World 2'],
            self._get_event_titles_from_block(browser)
        )

    @browsing
    def test_block_prevents_path_and_current_context(self, browser):
        """
        This test makes sure that a block cannot be configured to
        have a "filter by path" and a "filter by current context" at the
        same time.
        """
        event_folder = create(Builder('event folder').titled(u'Event Folder'))

        browser.login()

        # Note that a default event listing block is created
        # automatically in the event folder. Let's get the block.
        block = event_folder.listFolderContents(
            contentFilter={'portal_type': 'ftw.events.EventListingBlock'}
        )[0]

        # Edit the existing block.
        browser.visit(block, view='edit.json')
        response = browser.json
        browser.open_html(response['content'])
        browser.fill({
            'Title': u'Not relevant for this test',
            'Limit to path': '/'.join(event_folder.getPhysicalPath()),
            'Limit to current context': True,
        })
        browser.find_button_by_label('Save').click()
        response = browser.json
        browser.open_html(response['content'])

        # Make sure that the form refuses to save.
        self.assertEqual(
            ['There were some errors.'],
            statusmessages.error_messages()
        )
        self.assertIn(
            'You cannot filter by path and current context at the same time.',
            browser.css('.error').text
        )

    @browsing
    def test_block_filter_by_current_context(self, browser):
        """
        This test makes sure the block filters by the current context.
        """
        page1 = create(Builder('sl content page').titled(u'Content Page 1'))
        event_folder1 = create(Builder('event folder')
                               .titled(u'Event Folder 1')
                               .within(page1))
        create(Builder('event page').titled(u'Hello World 1').within(event_folder1))

        page2 = create(Builder('sl content page').titled(u'Content Page 2'))
        event_folder2 = create(Builder('event folder')
                               .titled(u'Event Folder 2')
                               .within(page2))
        create(Builder('event page').titled(u'Hello World 2').within(event_folder2))

        block = create(Builder('event listing block')
                       .within(page2)
                       .titled(u'Not relevant for this test'))

        browser.login()

        # Make sure that the filter by context is active by default.
        browser.visit(page2)
        self.assertEqual(
            ['Hello World 2'],
            self._get_event_titles_from_block(browser),
        )

        # Tell the block to no longer filter by the current context.
        browser.login().visit(block, view='edit.json')
        response = browser.json
        browser.open_html(response['content'])
        browser.fill({
            'Limit to current context': False,
        })
        browser.find_button_by_label('Save').click()

        # Make sure the title is no longer rendered.
        browser.visit(page2)
        self.assertEqual(
            ['Hello World 1', 'Hello World 2'],
            self._get_event_titles_from_block(browser),
        )

    @browsing
    def test_block_crops_description(self, browser):
        """
        This test makes sure that the block crops the description of the event
        page to the default length (note that the description is shown by default
        but can be disabled).
        """
        page = create(Builder('sl content page').titled(u'Content Page 1'))
        event_folder = create(Builder('event folder').titled(u'Event Folder 1').within(page))
        create(Builder('event page')
               .titled(u'Hello World 1')
               .having(description=u"This description must be longer than "
                                   u"50 characters so we are able to test "
                                   u"if it will be cropped.")
               .within(event_folder))
        create(Builder('event listing block')
               .within(page)
               .titled(u'This is a EventListingBlock'))

        browser.login().visit(page)
        self.assertEqual(
            ['This description must be longer than 50 ...'],
            browser.css('.sl-block.ftw-events-eventlistingblock .description').text
        )

    @browsing
    def test_block_renders_entire_description(self, browser):
        """
        This test makes sure that the block renders the entire description
        if configured to do so.
        """
        page = create(Builder('sl content page').titled(u'Content Page 1'))
        event_folder = create(Builder('event folder').titled(u'Event Folder 1').within(page))
        create(Builder('event page')
               .titled(u'Hello World 1')
               .having(description=u"This description must be longer than "
                                   u"50 characters so we are able to test "
                                   u"if it will be cropped.")
               .within(event_folder))
        create(Builder('event listing block')
               .within(page)
               .having(description_length=0)
               .titled(u'This is a EventListingBlock'))

        browser.login().visit(page)
        self.assertEqual(
            [
                'This description must be longer than 50 characters '
                'so we are able to test if it will be cropped.'
            ],
            browser.css('.sl-block.ftw-events-eventlistingblock .description').text
        )

    @browsing
    def test_block_does_not_render_description(self, browser):
        """
        This test makes sure that the block does not render the description
        if configured to do so.
        """
        page = create(Builder('sl content page').titled(u'Content Page'))
        event_folder = create(Builder('event folder').titled(u'Event Folder').within(page))
        create(Builder('event page')
               .titled(u'Hello World')
               .having(description=u"The description.")
               .within(event_folder))
        create(Builder('event listing block')
               .within(page)
               .having(show_description=False)
               .titled(u'This is a EventListingBlock'))

        browser.login().visit(page)
        self.assertEqual(
            [],
            browser.css('.sl-block.ftw-events-eventlistingblock .description').text
        )

    @browsing
    def test_block_filters_by_subject(self, browser):
        """
        This test makes sure that the block filters event pages by subject.
        """
        page = create(Builder('sl content page').titled(u'Content Page'))
        event_folder = create(Builder('event folder').titled(u'Event Folder').within(page))
        create(Builder('event listing block')
               .within(page)
               .having(subjects=['Hans'])
               .titled(u'This is a EventListingBlock'))
        with freeze(datetime.datetime(2010, 7, 1)):
            create(Builder('event page')
                   .titled(u'Event of Hans')
                   .having(subjects=['Hans'])
                   .starting(datetime.datetime(2010, 7, 2))
                   .ending(datetime.datetime(2010, 7, 2))
                   .within(event_folder))
            create(Builder('event page')
                   .titled(u'Event of Peter')
                   .having(subjects=['Peter'])
                   .starting(datetime.datetime(2010, 7, 2))
                   .ending(datetime.datetime(2010, 7, 2))
                   .within(event_folder))

            browser.login()
            browser.visit(page)
            self.assertEqual(
                ['Event of Hans'],
                self._get_event_titles_from_block(browser)
            )

    @browsing
    def test_block_limits_quantity(self, browser):
        page = create(Builder('sl content page').titled(u'Content Page'))
        event_folder = create(Builder('event folder').titled(u'Event Folder').within(page))
        quantity = 1
        create(Builder('event listing block')
               .within(page)
               .having(quantity=quantity)
               .having(show_title=False)
               .titled(u'This is a EventListingBlock'))

        with freeze(datetime.datetime(2010, 7, 1)):
            create(Builder('event page')
                   .titled(u'Event of Hans')
                   .starting(datetime.datetime(2010, 7, 2))
                   .ending(datetime.datetime(2010, 7, 2))
                   .within(event_folder))
            create(Builder('event page')
                   .titled(u'Event of Peter')
                   .starting(datetime.datetime(2010, 7, 2))
                   .ending(datetime.datetime(2010, 7, 2))
                   .within(event_folder))

            browser.login()
            browser.visit(page)
            self.assertEqual(
                quantity,
                len(self._get_event_titles_from_block(browser))
            )

    @browsing
    def test_block_renders_link_to_more_items(self, browser):
        page = create(Builder('sl content page')
                      .titled(u'Content Page'))
        event_folder = create(Builder('event folder').
                              titled(u'Event Folder')
                              .within(page))
        create(Builder('event page')
               .titled(u'Event')
               .within(event_folder))
        block = create(Builder('event listing block')
                       .within(page)
                       .having(show_more_items_link=True)
                       .titled(u'This is a EventListingBlock'))

        browser.login()

        # Make sure the link is there.
        browser.visit(page)
        self.assertEqual(
            'http://nohost/plone/content-page/this-is-a-eventlistingblock/events',
            browser.find('More Items').attrib['href']
        )

        # Tell the block to no longer render the link
        browser.login().visit(block, view='edit.json')
        response = browser.json
        browser.open_html(response['content'])
        browser.fill({
            'Show link to more items': False,
        })
        browser.find_button_by_label('Save').click()

        # Make sure the link is no longer there.
        browser.visit(page)
        self.assertIsNone(browser.find('More Items'))

    @browsing
    def test_block_renders_link_to_more_items_with_custom_label(self, browser):
        """
        This test makes sure that the link to thew view which renders more items
        has a custom label.
        """
        page = create(Builder('sl content page')
                      .titled(u'Content Page'))
        event_folder = create(Builder('event folder').
                              titled(u'Event Folder')
                              .within(page))
        create(Builder('event page')
               .titled(u'Event')
               .within(event_folder))
        create(Builder('event listing block')
               .within(page)
               .having(show_more_items_link=True)
               .having(more_items_link_label=u'Show me more')
               .titled(u'This is a EventListingBlock'))

        browser.login()

        # Make sure the link is there.
        browser.visit(page)
        self.assertEqual(
            'http://nohost/plone/content-page/this-is-a-eventlistingblock/events',
            browser.find('Show me more').attrib['href']
        )

    def test_syndication_is_enabled_by_default_on_block(self):
        folder = create(Builder('event folder'))

        # Note that a default event listing block is created
        # automatically in the event folder.
        block = folder.listFolderContents(
            contentFilter={'portal_type': 'ftw.events.EventListingBlock'}
        )[0]

        self.assertTrue(
            IFeedSettings(block).enabled
        )
        self.assertEqual(
            ('RSS', 'rss.xml', 'atom.xml'),
            IFeedSettings(block).feed_types
        )

    @browsing
    def test_block_renders_location(self, browser):
        event_folder = create(Builder('event folder'))
        event = create(Builder('event page')
                       .titled(u'My Event')
                       .having(location_title='Infinite Loop 1')
                       .within(event_folder))
        browser.login()

        # Make sure the location is rendered.
        browser.open(event_folder)
        self.assertEqual(
            ['Infinite Loop 1'],
            browser.css('.event-row .byline .location').text
        )

        # Empty the location and make sure it is no longer rendered.
        browser.visit(event, view='edit')
        browser.fill({'Location: title': u''}).submit()
        browser.open(event_folder)
        self.assertEqual(
            [],
            browser.css('.event-row .byline .location')
        )

    @browsing
    def test_block_does_not_render_past_events(self, browser):
        page = create(Builder('sl content page'))
        event_folder = create(Builder('event folder').titled(u'Event Folder').within(page))
        create(Builder('event listing block')
               .within(page)
               .titled(u'Event listing block'))

        with freeze(datetime.datetime(2010, 7, 1, 15, 0, 0)):
            create(Builder('event page')
                   .titled(u'Past event')
                   .starting(datetime.datetime(2010, 1, 1))
                   .ending(datetime.datetime(2010, 1, 1))
                   .within(event_folder))
            create(Builder('event page')
                   .titled(u'Today event')
                   .starting(datetime.datetime(2010, 7, 1, 12, 0, 0))
                   .ending(datetime.datetime(2010, 7, 1, 18, 0, 0))
                   .within(event_folder))
            create(Builder('event page')
                   .titled(u'Future event')
                   .starting(datetime.datetime(2010, 7, 2))
                   .ending(datetime.datetime(2010, 7, 2))
                   .within(event_folder))
            create(Builder('event page')
                   .titled(u'Event which ended a few hours ago')
                   .starting(datetime.datetime(2010, 7, 1, 9, 0, 0))
                   .ending(datetime.datetime(2010, 7, 1, 14, 0, 0))
                   .within(event_folder))

            browser.login()
            browser.visit(page)
            self.assertEqual(
                ['Event which ended a few hours ago', 'Today event', 'Future event'],
                self._get_event_titles_from_block(browser)
            )



from ftw.simplelayout.interfaces import IPageConfiguration
from plone import api
from plone.uuid.interfaces import IUUID
import transaction


def create_page_state(obj, block):
    page_state = {
        "default": [
            {
                "cols": [
                    {
                        "blocks": [
                            {
                                "uid": IUUID(block)
                            }
                        ]
                    }
                ]
            },
        ]
    }
    page_config = IPageConfiguration(obj)
    page_config.store(page_state)
    transaction.commit()


def set_allow_anonymous_view_about(state):
    site_props = api.portal.get_tool(name='portal_properties').site_properties
    site_props.allowAnonymousViewAbout = state
    transaction.commit()



from ftw.builder import Builder
from ftw.builder import create
from ftw.events.tests import FunctionalTestCase
from ftw.testbrowser import browsing
from plone import api


class TestEventListing(FunctionalTestCase):

    def setUp(self):
        super(TestEventListing, self).setUp()
        self.grant('Manager')

    @browsing
    def test_event_listing(self, browser):
        """
        This test makes sure the event listing view renders the events.
        """
        event_folder = create(Builder('event folder')
                              .titled(u'Not relevant in this test'))
        create(Builder('event page')
               .titled(u'My Event')
               .within(event_folder))

        browser.login()

        # Get the event folder's event listing block which has been created automatically.
        block = api.content.find(
            portal_type='ftw.events.EventListingBlock',
            within=event_folder
        )[0].getObject()

        # Make sure the view renders the events.
        browser.visit(block, view='@@events')
        self.assertEqual(
            ['My Event'],
            browser.css('.event-row .title').text
        )

    @browsing
    def test_custom_event_listing_title(self, browser):
        page = create(Builder('sl content page')
                      .titled(u'Not relevant in this test'))
        event_folder = create(Builder('event folder')
                              .titled(u'Not relevant in this test')
                              .within(page))
        create(Builder('event page')
               .titled(u'Not relevant in this test')
               .within(event_folder))
        block = create(Builder('event listing block')
                       .within(page)
                       .titled(u'Not relevant in this test'))

        browser.login()

        # A fallback title must be rendered on the view by default.
        browser.visit(block, view='@@events')
        self.assertEqual(
            ['Events'],
            browser.css('.documentFirstHeading').text
        )

        # Configure a custom title.
        browser.login().visit(block, view='edit.json')
        response = browser.json
        browser.open_html(response['content'])
        browser.fill({
            'Title of the view behind the "more items" link': u'All my events',
        })
        browser.find_button_by_label('Save').click()

        # Now the view must render the custom title.
        browser.visit(block, view='@@events')
        self.assertEqual(
            ['All my events'],
            browser.css('.documentFirstHeading').text
        )

    @browsing
    def test_event_listing_renders_location(self, browser):
        event_folder = create(Builder('event folder'))
        event = create(Builder('event page')
                       .titled(u'My Event')
                       .having(location_title='Infinite Loop 1',
                               location_street='Hamburgstrasse 3743x',
                               location_zip='12345',
                               location_city='Hamburg')
                       .within(event_folder))
        browser.login()

        # Get the event folder's event listing block which has been created automatically.
        block = api.content.find(
            portal_type='ftw.events.EventListingBlock',
            within=event_folder
        )[0].getObject()

        # Make sure the location is rendered.
        browser.visit(block, view='@@events')
        self.assertEqual(
            ['Infinite Loop 1, Hamburgstrasse 3743x, 12345 Hamburg'],
            browser.css('.event-row .byline .location').text
        )

        # Empty the location and make sure it is no longer rendered.
        browser.visit(event, view='edit')
        browser.fill({'Location: title': u'',
                      'Location: street and number': u'',
                      'Location: ZIP code': u'',
                      'Location: city': u''}).submit()
        browser.visit(block, view='@@events')
        self.assertEqual(
            [],
            browser.css('.event-row .byline .location')
        )



from DateTime import DateTime
from ftw.events.behaviors.location import ILocationFields
from ftw.events.behaviors.mopage import IMopageModificationDate
from htmlentitydefs import name2codepoint as n2cp
from plone.app.dexterity.behaviors.metadata import ICategorization
from plone.event.interfaces import IRecurrenceSupport
from plone.uuid.interfaces import IUUID
from Products.CMFCore.utils import getToolByName
from Products.Five.browser import BrowserView
from zExceptions import BadRequest
import lxml.html
import math
import re
import urllib
import urlparse


def normalize_whitespace(text, replacement=' '):
    text = re.sub(r'^\s+', replacement, text)
    text = re.sub(r'\s+$', replacement, text)
    return text


def normalize_join(*parts):
    return normalize_whitespace(''.join([part or '' for part in parts]))


def remove_node(node):
    parent = node.getparent()
    if parent is None:
        return

    position = parent.index(node)

    if position == 0:
        parent.text = normalize_join(parent.text, node.tail)
    else:
        sibling = parent[position - 1]
        sibling.tail = normalize_join(sibling.tail, node.tail)

    parent.remove(node)


def crop(length, text):
    if not text:
        return text

    if isinstance(text, str):
        text = text.decode('utf-8')
    if len(text) > length:
        text = text[:length - 5] + ' ...'

    return text


def decode_entities(text):
    """
    Decodes html entities and xml entities.
    """
    entity_re = re.compile("&(#?)(\d{1,5}|\w{1,8});")

    def substitute_entity(match):
        ent = match.group(2)
        if match.group(1) == "#":
            return unichr(int(ent))

        else:
            cp = n2cp.get(ent)
            if cp:
                return unichr(cp)
            else:
                return match.group()

    return entity_re.subn(substitute_entity, text)[0]


class MopageEvents(BrowserView):
    """Documentation: http://doc.mopage.ch/ext/XML_Schnittstelle/2_XML_Aufbau
    """

    def __call__(self):
        self.request.RESPONSE.setHeader('Cache-Control', 'no-store')
        self.request.RESPONSE.setHeader('Content-Type',
                                        'text/xml;charset=utf-8')
        return super(MopageEvents, self).__call__()

    def import_node_attributes(self):
        attrs = {'export_time': self.normalize_date(DateTime())}
        for name in ('partner', 'partnerid', 'passwort', 'importid',
                     'vaterobjekt'):
            attrs[name] = self.request.form.get(name, None)
        return attrs

    def items(self):
        brains = self.apply_pagination(self.get_brains())
        return map(self.brain_to_item, brains)

    def get_brains(self):
        catalog = getToolByName(self.context, 'portal_catalog')
        return catalog(self.get_query())

    def apply_pagination(self, brains):
        per_page = self.get_int_param_from_request('per_page', 100)
        page = self.get_int_param_from_request('page', 1)
        last = page * per_page
        first = last - per_page
        batch = brains[first:last]

        links = []
        if page > 1:
            links.append(self.build_pagination_link('first', page=1))
            links.append(self.build_pagination_link('prev', page=page - 1))

        if len(brains) > last:
            links.append(self.build_pagination_link('next',
                                                    page=int(page + 1)))
            links.append(self.build_pagination_link(
                'last', page=int(math.ceil(len(brains) / float(per_page)))))

        if links:
            self.request.RESPONSE.setHeader('Link', ','.join(links))

        return batch

    def get_query(self):
        return {
            'object_provides': 'ftw.events.interfaces.IEventPage',
            'sort_on': 'start',
            'sort_order': 'reverse',
            'path': '/'.join(self.context.getPhysicalPath())}

    def brain_to_item(self, brain):
        obj = brain.getObject()
        image_url = self.get_lead_image_url(obj)

        if image_url and len(image_url) > 255:
            # sorry, not supported.
            image_url = None

        textlead = crop(100, obj.Description() or '')
        if textlead:
            textlead = u'<![CDATA[{}]]>'.format(textlead)

        subjects = getattr(ICategorization(obj, None), 'subjects', ())
        modified_date = IMopageModificationDate(obj).get_date()
        occurences = IRecurrenceSupport(obj).occurrences()

        location = {}
        if ILocationFields.providedBy(obj):
            storage = ILocationFields(obj)
            # All fields wich are "required" by the API must have a value
            # in order to have a location.
            if storage.location_title \
               and storage.location_street \
               and storage.location_zip \
               and storage.location_city:
                location.update({
                    'title': storage.location_title,
                    'street': storage.location_street,
                    'zip': storage.location_zip,
                    'city': storage.location_city,
                })

        return {'title': crop(100, brain.Title),
                'modified_date': self.normalize_date(modified_date),
                'occurences': occurences,
                'uid': IUUID(obj),
                'url': brain.getURL(),
                'textlead': textlead,
                'image_url': image_url,
                'subjects': map(lambda subject: crop(100, subject), subjects),
                'location': location,
                'obj': obj}

    def normalize_date(self, date):
        if not date:
            return ''

        if DateTime(date) > DateTime(2100, 1, 1):
            return None

        return date.strftime('%Y-%m-%d %H:%M:%S')

    def cleanup_body_html(self, html):
        """We have HTML with simplelayout structure, but this is too noisy
        for the import.
        We therefore convert the HTML to text and back to HTML.
        We also need to make sure that the result is less than 10000 long.
        """

        doc = lxml.html.fromstring(html)
        map(remove_node, doc.xpath(
            '//*[contains(concat(" ", normalize-space(@class), " "), '
            '" hiddenStructure ")]'))
        html = lxml.etree.tostring(doc, pretty_print=True)

        portal_transforms = getToolByName(self.context, 'portal_transforms')
        text = decode_entities(portal_transforms.convertToData(
            'text/x-web-intelligent',
            html,
            mimetype='text/html').strip())

        # body limit is 10000.
        # We have text here, but will convert it to HTML, so it will be larger
        # and we need to crop more to compensate.
        for attempt in range(100):
            text = crop(10000 - (len(text) * 0.1), text)
            html = portal_transforms.convertToData(
                'text/html',
                text,
                mimetype='text/x-web-intelligent').strip()

            if len(html) < 10000:
                return u'<![CDATA[' + html + u']]>'

        return u'<![CDATA[cropping error]]>'

    def get_lead_image_url(self, events):
        scale = events.restrictedTraverse('@@leadimage').get_scale()
        return scale and scale.url or None

    def get_int_param_from_request(self, name, default):
        value = int(self.request.get(name, default))
        if value < 1:
            raise BadRequest('Invalid "{}" parameter value.'.format(name))
        return value

    def build_pagination_link(self, rel, **kwargs):
        params = self.request.form.copy()
        params.update(kwargs)
        parts = list(urlparse.urlparse(self.request.ACTUAL_URL))
        parts[4] = urllib.urlencode(params)
        url = urlparse.urlunparse(parts)
        return '<{}>; rel="{}"'.format(url, rel)
